/*
 * Created on 08.03.2003
 *
 * Jrgen Seitz 	-	Software nach Mass
 */
package stammdaten;

import interfaces.Anschrift;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import exceptions.DuplicateKeyException;

/**
 * @author jrgen seitz
 *
 * einfache Klasse zum Speichern von Adressdaten
 */
public class StandardAdresse {
	/**
	 * @author jrgen seitz
	 *
	 * einfache Klasse zum Speichern von Anschriftdaten
	 */
	public static class StandardAnschrift{
		/**
		 * 
		 */
		public StandardAnschrift(String strasse, String hausnummer, String postleitzahl, String ort, String land) {
			this.strasse = strasse;
			this.hausNummer = hausnummer;
			this.postleitzahl = postleitzahl;
			this.ort = ort;
			this.land = land;		
		}
		/**
		 * @return String
		 * gibt die VersionsNummer als String zurck
		 * dadurch, dass die Methode als static definiert wurde,
		 * kann sie aufgerufen werden ohne instanziert werden zu mssen !
		 */
		private static String getVersion() {
			return "1b";
		}
		
		private String hausNummer;
		private String land;
		private String ort;
		private String postleitzahl;
		private String strasse;
	
		/**
		 * @return String
		 * gibt die Hausnummer als String zurck	 */
		public String getHausNummer() {
			return hausNummer;
		}
	
		/**
		 * @return String
		 * gibt das Land als String zurck
		 */
		public String getLand() {
			return land;
		}
	
		/**
		 * @return String
		 * gibt den Ort als String zurck
		 */
		public String getOrt() {
			return ort;
		}
	
		/**
		 * @return String
		 * gibt die Postleitzahl als String zurck
		 */
		public String getPostleitzahl() {
			return postleitzahl;
		}
	
		/**
		 * @return String
		 * gibt die Strasse als String zurck
		 */
		public String getStrasse() {
			return strasse;
		}
	
		/**
		 * @see java.lang.Object#toString()	 * 
		 * @return String
		 * gibt die komplette StandardAnschrift als String zurck
		 */
		public String toString() {
			return this.strasse.toString() + " " + hausNummer.toString() + ", " + postleitzahl.toString() + " " + ort.toString() + ", " + land.toString();
		}
	
	}
	private static int anschriftCounterClass = 0; 	// wird bei neuer Instanz des Objektes nicht zurckgesetzt !
	private int anschriftCounterInstance = 0; 		// wird bei jeder neuen Instanz des Objektes wieder auf 0 zurckgesetzt !

	protected Map anschriften = new HashMap();
	private Integer id;
	private String name;
	private String vorname;
	
	/**
	 * Konstruktor zum Instanzieren einer neuen Adresse mit allen bentigten Werten
	 */
	public StandardAdresse(Integer id, String name, String vorname, StandardAnschrift anschrift) {
		this.id = id;
		this.name = name;
		this.vorname = vorname;
		try {
			addAnschrift(Anschrift.DEFAULT_ANSCHRIFT, anschrift);
		} catch (DuplicateKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Fgt eine andere Anschrift unter einem bestimmten key zur Adresse hinzu
	 * @param key 
	 * @param anschrift 
	 */
	public void addAnschrift(String key, StandardAnschrift anschrift) 
	// Diese Methode wirft (throws) nur DuplicateKeyException !
	throws DuplicateKeyException {
		if (anschriften.containsKey(key))
			// Key schon enthalten, also Exception werfen (throw)
			throw new DuplicateKeyException("Doppelter Key : " + key);
		
		// Diese Anweisungen werden im Fehlerfall nicht mehr ausgefhrt !!	
		anschriften.put(key, anschrift);
		anschriftCounterClass = ++anschriftCounterClass;
		anschriftCounterInstance = ++anschriftCounterInstance;
		System.out.println("classCounter : " + anschriftCounterClass + " instanceCounter : " + anschriftCounterInstance);
	}

	/**
	 * @return Integer 
	 * gibt die ID der Adresse als Integer zurck
	 */
	public Integer getId() {
		return id;
	}

	/**
	 * @return StandardAnschrift
	 * gibt die Anschrift der Adresse als StandardAnschrift zurck
	 */
	public StandardAnschrift getAnschrift(String key) {
		return (StandardAnschrift)anschriften.get(key);
	}

	/**
	 * @return String
	 * gibt den Namen der Adresse als String zurck
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return String
	 * gibt den Vornamen der Adresse als String zurck
	 */
	public String getVorname() {
		return vorname;
	}

	/**
	 * @see java.lang.Object#toString()
	 * gibt die komplette StandardAdresse als String zurck
	 */
	protected String getAnschriftenString() {
		String anschriftString = "";
		for (Iterator iterator = anschriften.keySet().iterator(); iterator.hasNext();) {
			String anschriftsTyp = (String) iterator.next();
			anschriftString = anschriftString + ", " + anschriftsTyp + " : " + getAnschrift(anschriftsTyp);
		}
		return anschriftString;
	}

	/**
	 * @see java.lang.Object#toString()
	 * gibt die komplette StandardAdresse als String zurck
	 */
	public String toString() {
		return "ID: " + getId() + ", " + getVorname() + " " + getName() + getAnschriftenString();
	}
}
